function [L, M, Rel_error] = rpca_partial(A, r, alpha, params)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    RPCA Nonconvex Feasibility Algotithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A Nonconvex Projection method for Robust PCA, Aritra Dutta, Filip
% Hanzely, and Peter Richtrik, Thirty-Third AAAI Conference on Artificial
% Intelligence, 2019. 
%%%%%%%%%%%
% A : A data matrix to be decomposed into a low-rank matrix L and a sparse
% matrix M. Unobserved entries are represented as zeros.
% r : Target rank
% alpha : An upper bound of max sparsity over the columns/rows of S
% params : parameters for the algorithm
%   .max_iter : Maximum number of iterations (default 30)
%   .tol : Desired Frobenius norm error (default 2e-4)
%%%%%%%%%%%
% Output:: L : Low rank matrix of rank r
%          M : Sparse matrix of sparsity alpha*m*n
%  Rel_error : ||A-L-M||_F/||A||_F

tic
% Default parameter settings
maxIter   = 30;
thresh    = 2e-4;
gamma     = 1;
incoh     = 5;

%%%% Read paramter settings
if isfield(params,'gamma')      
    gamma = params.gamma; 
end
if isfield(params,'incoh')      
    incoh = params.incoh; 
end
if isfield(params,'max_iter')   
    maxIter = params.max_iter; 
end
if isfield(params,'thresh')     
    thresh= params.thresh; 
end

% Add Library paths
addpath PROPACK;
addpath MinMaxSelection;
addpath Functions;
addpath bksvd-master;

%%%%%% Setting up %%%%%%%

Norm_of_A = norm(A,'fro');
[d1, d2] = size(A);

is_sparse  = issparse(A);
if is_sparse
    [I, J, A_vec] = find(A);
    n = length(A_vec);
    obs_ind = sub2ind([d1,d2], I, J);
    col = [0; find(diff(J)); n];
    p = n/d1/d2;
    if p>0.9
        is_sparse = 0;
        A = full(A);
    end
else
    p = 1;
end

%%%%% Initialization %%%%%%%
L_old = zeros(size(A));
M_old = zeros(size(A));
Rel_error = zeros(1,maxIter); converged = 0; %%(When showing converegcne of the algorithm)
iter = 0;

%%%% Iteration %%%%%%%%%
for i = 1:maxIter
    % while ~converged
    iter  = iter+1;
    
    %%% Projection on the convex set::: A=L+S
    L_temp = L_old/2-M_old/2+A/2;
    M_temp = M_old/2-L_old/2+A/2;
    
    %%%% Low rank Projection--- Pick your favorite SVD. Generally, bksvd is faster and more recent %%%%%%
    
    %[U, Sigma, V] = lansvd(L_temp,r,'L');
    if p==1
        [U,Sigma,V] = bksvd(L_temp,r,1);
    else
        [U, Sigma, V] = lansvd(L_temp,r,'L');
    end
    
    L = U*Sigma*V';
    
    %%%% Sparse Projection %%%%%%
    M = Tproj_partial_RPCAF(M_temp, gamma*p*alpha, gamma*p*alpha);
    
    Residual = A-L-M;
    Rel_error(1,iter) = norm(Residual,2)/Norm_of_A;
    
    L_old = L;
    M_old = M;
    
    %%%%  Convergence criteria %%%%
         if (Rel_error(iter) <= thresh ||iter>maxIter)
             converged = 1;
         end
    
end
toc







